import _JSXStyle from "styled-jsx/style";
import React from "react";
import { Button, Ref } from "semantic-ui-react";
import { CalendarIcon, CancelIcon } from "@orbit-ui/icons";
import { KEYS, isNullOrEmpty, mergeClasses, withHandlerProxy } from "@orbit-ui/react-components-shared";
import { PureComponent, createRef } from "react";
import { ResizeObserver } from "./resize-observer";
import { bool, func, node, string } from "prop-types";
import { isNil } from "lodash";
import cx from "classnames";
export class DatePickerTextboxInput extends PureComponent {
  constructor(...args) {
    super(...args);
    this._containerRef = createRef();
    this._clearButtonRef = createRef();
    this._containerResizeObserver = null;

    this.handleContainerSizeChange = entries => {
      const {
        onSizeChange
      } = this.props;

      if (!isNil(onSizeChange)) {
        // The native chrome implementation doesn't currently support the "border-box" specs. Therefore, we rely on "getBoundingClientRect"
        // for the value when a size changed is detected.
        // Specs available here: https://drafts.csswg.org/resize-observer-1/
        const dimensions = entries[0].target.getBoundingClientRect();
        onSizeChange({
          width: dimensions.width,
          height: dimensions.height
        });
      }
    };

    this.handleClick = event => {
      const {
        onClick,
        onOpen,
        onClose,
        allowClear,
        disabled,
        open
      } = this.props;
      let canPropagate = true;

      if (allowClear) {
        canPropagate = !this._clearButtonRef.current.contains(event.target);
      }

      if (canPropagate) {
        if (!isNil(onClick)) {
          onClick(event, this.props);
        }

        if (!disabled) {
          if (!open) {
            onOpen(event, this.props);
          } else {
            onClose(event, this.props);
          }
        }
      }
    };

    this.handleKeyDown = event => {
      const {
        onKeyDown,
        onOpen,
        onClear,
        disabled,
        open
      } = this.props;

      if (!disabled) {
        const key = event.keyCode;

        if (key === KEYS.space || key === KEYS.enter) {
          if (key === KEYS.space) {
            event.preventDefault();
          }

          if (!open) {
            onOpen(event, this.props);
          }
        } else if (key === KEYS.esc) {
          if (!open) {
            onClear(event, this.props);
          }
        }
      }

      if (!isNil(onKeyDown)) {
        onKeyDown(event, this.props);
      }
    };

    this.handleFocus = withHandlerProxy(this, "onFocus");
    this.handleBlur = withHandlerProxy(this, "onBlur");
    this.handleClearButtonClick = withHandlerProxy(this, "onClear");
  }

  componentDidMount() {
    this._containerResizeObserver = new ResizeObserver(this.handleContainerSizeChange);

    this._containerResizeObserver.observe(this._containerRef.current);
  }

  componentWillUnmount() {
    this._containerResizeObserver.disconnect();
  }

  isPlaceholder() {
    const {
      value
    } = this.props;
    return isNullOrEmpty(value);
  }

  getCssClasses() {
    const {
      disabled,
      open,
      className
    } = this.props;
    return mergeClasses("input pv3 ph4 ba outline-0 f6 h9 br2 flex items-center", open ? "b--marine-600 marine-600" : "b--cloud-200 marine-200", !this.isPlaceholder() && "marine-600", !disabled ? "hover-b--marine-600 hover-marine-600 pointer" : "bg-cloud-100 cloud-400 crsr-not-allowed", className);
  }

  renderIcon() {
    const {
      icon,
      disabledIcon,
      disabled
    } = this.props;
    return disabled ? disabledIcon : icon;
  }

  renderClearButton() {
    const {
      allowClear,
      clearIcon,
      disabled,
      open
    } = this.props;

    if (!allowClear) {
      return null;
    }

    return React.createElement("div", {
      className: cx({
        dn: this.isPlaceholder() || disabled || open
      })
    }, React.createElement(Ref, {
      innerRef: this._clearButtonRef
    }, React.createElement(Button, {
      circular: true,
      size: "tiny",
      primary: true,
      icon: true,
      className: "transparent",
      onClick: this.handleClearButtonClick,
      type: "button"
    }, clearIcon)));
  }

  render() {
    const {
      value,
      placeholder,
      disabled
    } = this.props;
    return React.createElement("div", {
      onClick: this.handleClick,
      onKeyDown: this.handleKeyDown,
      onFocus: this.handleFocus,
      onBlur: this.handlerBlur,
      tabIndex: disabled ? "-1" : "0",
      autoComplete: "off",
      disabled: disabled,
      ref: this._containerRef,
      className: "jsx-2462695752" + " " + (this.getCssClasses() || "")
    }, this.renderIcon(), React.createElement("span", {
      className: "jsx-2462695752" + " " + "flex-grow-1 ml2"
    }, this.isPlaceholder() ? placeholder : value), this.renderClearButton(), React.createElement(_JSXStyle, {
      id: "2462695752"
    }, [".input.jsx-2462695752:not(\"disabled\"){cursor:text;}", ".input.jsx-2462695752:focus{border:1px solid var(--marine-700);color:var(--marine-700);}"]));
  } // This method is part of the component external API.


  focus() {
    if (!isNil(this._containerRef.current)) {
      this._containerRef.current.focus();
    }
  }

}
DatePickerTextboxInput.propTypes = {
  value: string.isRequired,
  onOpen: func,
  onClose: func,
  onSizeChange: func,
  onClick: func,
  onKeyDown: func,
  // eslint-disable-next-line react/no-unused-prop-types
  onFocus: func,
  // eslint-disable-next-line react/no-unused-prop-types
  onBlur: func,
  // eslint-disable-next-line react/no-unused-prop-types
  onClear: func,
  allowClear: bool,
  placeholder: string,
  icon: node,
  clearIcon: node,
  disabledIcon: node,
  disabled: bool,
  open: bool,
  className: string
};
DatePickerTextboxInput.defaultProps = {
  allowClear: true,
  icon: React.createElement(CalendarIcon, {
    className: "w7 h7 fill-marine-700"
  }),
  clearIcon: React.createElement(CancelIcon, {
    className: "h3 w3"
  }),
  disabledIcon: React.createElement(CalendarIcon, {
    className: "w7 h7 fill-cloud-500"
  }),
  disabled: false,
  placeholder: "Pick a date"
};